/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xml.type.AnyType;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IMigrationService;
import org.fortiss.tooling.kernel.service.base.IObjectAware;

/**
 * Interface for the migration provider implementations.
 * <P>
 * migration provider extensions are handled by {@link IMigrationService}.
 * 
 * @author mou
 */
public interface IMigrationProvider extends IObjectAware<ITopLevelElement> {

	/**
	 * Determines whether migration provider is needed for the given
	 * model element.
	 * 
	 * The parameter "unknownFeatures" returns a map indicating the features that are not recognized
	 * in the model. This can be useful to detect features coming from old models and can be then
	 * translated to the new model by a migrator.
	 */
	boolean needMigration(ITopLevelElement modelElement, Map<EObject, AnyType> unknownFeatures);

	/**
	 * Applies the provider to the given element.
	 * 
	 * The parameter "unknownFeatures" returns a map indicating the features that are not recognized
	 * in the model. This can be useful to detect features coming from old models and can be then
	 * translated to the new model by a migrator.
	 * 
	 * @return all features which could not be migrated.
	 */
	Map<EObject, AnyType> migrate(ITopLevelElement modelElement,
			Map<EObject, AnyType> unknownFeatures);
}
